/*
 * Project Name:
                        MINI OSCILLOSCOPIO CON DISPLAY NOKIA 3310 (84x48)
                        PIC16F1825
                        Alim. 5V (funziona male se inferiore a 5 V)

 * description file:
                       Oscillo_Nokia_18F2331.c       main e funzioni oscilloscopio
                       Nokia3310_Lib_.c              libreria per GLCD KS0713
                       Init_CPU.c                    Inizializzazione micro
                       Oscillo_Nokia_bmp_1.h         immagine retticolo

  * Revision History:
                       11/01/2019: - Init
                       12/01/2019: - Max campionameto: 316Ksps. Poco!
                       13/09/2020 - Con "while (GO_bit){}": 442 Ksps,
                                    con "Delay_semple" e 6 "nop": 610 Ksps.
                                    con "Delay_semple" e 1 "nop": 1 Msps. In questo caso
                                    la schermata risulta non corretta (il segnale non
                                    viene ricreato in maniera corretta).

 * Test configuration:
     MCU:             PIC16F1825
     Scheme:
     Board:           Oscillo_PIC16 (13/09/2020)
     Oscillator:      INT+PLL, 32,000MHz
     Ext. Modules:    -
     SW:              mikroC pro

 * NOTES:
      GLCD: pin 1:  BOARD: +3,3V  (V.alimentazione)
            pin 2:         RC4    (SCK)
            pin 3:         RC5    (SDI)
            pin 4:         RC6    (D_C)
            pin 5:         GND    (SCE -> ENABLE)
            pin 6:         GND    (GND)
            pin 7:         1uF    (CONDENSATORE CON PIN - A MASSA)
            pin 8:         RC7    (RESET)

      INPUT ANALOGICO: RA2
      OUT PWM:         RC1
      INPUT INT:       RA0
      LED:             RC0

      Nokia GLCD bit ADC: sulla parte alta del display max. 151 bit
                          sulla parte bassa min.            104 bit

        CENTRALE UNIVERSALE 3B
// --- LED (OUTPUT)
 sbit LED_1 at LATA5_bit;   // OUT 1
 sbit LED_2 at LATC0_bit;   // OUT 2
 sbit LED_3 at LATB7_bit;   // OUT 3
 sbit LED_4 at LATB6_bit;   // OUT 4
 sbit LED_5 at LATB5_bit;   // OUT 5
 sbit LED_6 at LATC2_bit;   // OUT 6
// --- Pulsanti (INPUT)
 sbit pul_1 at RC6_bit;   // IN 1
 sbit pul_2 at RC7_bit;   // IN 2
 sbit pul_3 at RB4_bit;   // IN 3
 sbit pul_4 at RC1_bit;   // IN 4
 sbit pul_5 at RA3_bit;   // IN 5
*/
//
 #include  "Init_PIC16F182X.c"       // Inizializzazione CPU
 #include  "Variabili.h"
 #include  "Lib_Utility.h"
 #include  "Nokia3310_Lib_.c"
 #include  "PGA_MCP6S9X.h"          // Libreria gestione PGA
 #include  "Utility_Oscillo_V2.c"
 #include  "ADC_Library.c"
//
 sbit pul1 at RA3_bit;   // IN 4  pul_up
 sbit pul2 at RA1_bit;   // IN 5  pul_dw
 sbit pul3 at RC0_bit;   // IN 4  pul_set
 sbit pul4 at RC2_bit;   // IN 5  pul_end
//
 sbit led  at LATA0_bit; // OUT
//
 sbit in_bf at RC1_bit;  // IN
// sbit trig  at RA2_bit;  // IN
 sbit pwm   at LATC1_bit;  // OUT
 sbit ce_pga at LATC3_bit; // OUT
 sbit sck    at LATC4_bit; // OUT
 sbit si_pga at LATA4_bit; // OUT
//
// *****************************************************************************
//
// *****************************************************************************
 void Init_Oscillo () {
//
     initlcd();       // Inizializzazione GLCD
//
      Write_Bitmap(2);
       Delay_msec(255);
        Delay_msec(255);
         Delay_msec(255);
          Delay_msec(255);
//

     clearram();
     Value_Div_Volt = 7;      // 1V
     Value_Div_Tempo = 4;     // 1ms
     FRONTE = 1;              // in salita
     M_trigger = 0 ;
    // CCPR1L = 128;            // Centro schermo
     Value_Trigger_Mode = 1;   // Trigger
     Select_Mark_Vert = 2;     // NO Marker verticale
     Select_Mark_Hor = 2;      // NO Marker orizzontale
     Calc_Delta_M_Vert ();
     Calc_Delta_M_Hor ();
     Adeg_Tempi ();
     Adeg_Trigger ();
     Adeg_Volt ();
//
    count_info = 0;
    Page_1_info ();       // Visualizza info
//
 }
// ---------------------------------------------------------- TEST
 void Clear_Camp() {
  unsigned short i;
//
      for(i=0; i<=100; i++) {
        sample_v1[i] = 151;   //120
      }
 }

// -------------------------------------------------------- INTERRUPT
void interrupt () {
// ----------------------- INTERRUPT INT0  // INTCON.INT0IE = 1;  // abilita INT0
                                           // INTCON.INT0IF = 0;
                                           // INTCON2.INTEDG0 = 1; // Fronte in salita
                                           // INTCON2.INTEDG0 = 0; // Fronte in discesa
   /*   if (INTCON.INT0IF) {
        M_INT0 = 1;           // Marker INT0
         INTCON.INT0IF = 0;
      } */
// ----------------------- INTERRUPT TMR6
      if (PIR3.TMR6IF) {
       PIR3.TMR6IF = 0;
        t6_1ms++;
         if (t6_1ms == camp_lung_ms && t6_100ms == camp_lung_100ms) M_t6 = 1;
          if (t6_1ms == 100) {
           t6_1ms = 0;
            t6_100ms ++;
          }

      }
 }
// --------------------------------------------------------
//                                                          MAIN
// --------------------------------------------------------
 void main() {
  unsigned short count_timeout, i;
//  unsigned int temp_16;
//
    Init_PIC();          // Inizilizzazione micro
     Delay_msec (100);
  //   Leggi_EEprom_page_1 ();                 // Inizializza i registri di pagina 1
//
       Delay_msec(100);
        initlcd();       // Inizilizzazione GLCD
         clearram();   
//
      //  Init_Oscillo ();
      
         Value_Div_Volt=5;//////////////////////////
         Value_Div_Tempo = 7;
         Value_Trigger_Mode = 1;
          Adeg_Trigger ();
           Adeg_Volt ();
            Adeg_Tempi ();
             ADC_Init_();
              Calc_Delta_M_Vert ();
               Calc_Delta_M_Hor ();
                PGA_Guadagno (guadagno_PGA); // inizializzazione PGA
                PGA_Guadagno (guadagno_PGA); // inizializzazione PGA

//
//--- Fronte in salita
//  C1INTP_bit = 1;
//   C1INTN_bit = 0;
// --- Fronte in discesa
       C1INTP_bit = 0;
        C1INTN_bit = 1;


//
    //////////////////////////////////////   Stop_Start_PWM1 (1);   // Start PWM1 (comparatore del trigger)
//

        Select_Mark_Vert = 2;     // NO Marker verticale
        Select_Mark_Hor = 2;      // NO Marker orizzontale
        FRONTE = 1;  // in salita
         Trigger = 10;
          M_trigger = 0 ;
           CCPR1L = 128;       // Centro schermo
            old_level = 0;
             DACCON1 = 0x10;   // 16 = 1/2 Vcc
            Info_page_1 (0);  // Visualizza info

//
//
   while (1) {
//
    if (M_camp_lungo_ON) goto  _loop_camp_lungo;  // CAMPIONAMENTO in tempo reale
//
      while (M_trigger == 0) {   // -------------------------TRIGGER   || Value_Trigger_Mode != 2
//
        if (Value_Trigger_Mode == 0) {         // TRIGGER RUN mode
         //INTCON.INT0IE = 0;  // Disabilita INT0
           goto _loop_RUN_mode;
        }
          else {
          // INTCON.INT0IE = 1;  // abilita INT0
          //  INTCON.INT0IF = 0;    // Abilita over-flow INT0
            PIR2.C1IF = 0;
             count_timeout = 0;   // Inizializzazione TIME-OUT
//
              if (FRONTE) {     // --------------------------------FRONTE IN SALITA
               C1INTP_bit = 1;
                C1INTN_bit = 0; // Fronte in salita
                while (PIR2.C1IF == 0) {
                 count_timeout ++;                  // TIME-OUT
                  if (count_timeout.F4) goto _loop_3 ;  // TIME-OUT
                   //if (PIR2.C1IF == 0)  goto _loop_1;
                 //   if (P_UP == 0 || P_DOWN == 0) Level_Trigger (); // Aggiorna e visualizza livello trigger  ////////////
                }
              }
//
               else {              // --------------------------------FRONTE IN DISCESA
                C1INTP_bit = 0;
                 C1INTN_bit = 1; // Fronte in discesa
                 while (PIR2.C1IF == 0){
                  count_timeout ++;                  // TIME-OUT
                   if (count_timeout.F4) goto _loop_3;
//
                  // if (PIR2.C1IF == 0)  goto _loop_2;
                  //   if (P_UP == 0 || P_DOWN == 0) Level_Trigger (); // Aggiorna e visualizza livello trigger   ///////////
                 }
               }
               
                PIR2.C1IF = 0;
// -------------------------------------------------- RUN mode
 _loop_RUN_mode:    // RUN mode
          if (Value_Div_Tempo == 1) {  // Campionamneto velose
           led=1;
            Sample_Fast();
             led=0;
              M_trigger = 1;  // Campionamenti eseguiti
          }

// ---- Campionamenti temporizzati
 //_Sample_Midle:   // -------------------- Campionamenti temporizzati
//
           else {
            led = 1;
             if (Value_Div_Tempo == 2) Sample_temp_rap();
              else Sample_temp();
               led =0;
                M_trigger = 1;  // Campionamenti eseguiti
           }
         } // ELSE
      } // while (M_trigger == 0)
 _loop_3:     // ------------------------time-out
//

// ----------------------------------------------------- VISUALIZZAZIONE CAMPIONAMENTO
//
// Area visualizzabile (GLCD 84x48): tra 104 e 152 bit di ADRESH (Y=48).
// Centro schermo: 128 bit (ADC).
//
// ------------------------------------------------------
      if (M_trigger) {
        ii = 0;
         while (ii <= info_on_off) {     // 59 o 80 colonne
          i = Sample_v1[ii];      // Read Sample
           i = 255-i;
            i >>= 2;
//
            if (ii == 0) old_point = i;
             else  {
               nokia_colonna_Bitmap(ii+1);                   // colonna retticolo
             //    if (i <= 104) i = 104;
             //     if (i >= 151) i = 151;
             //      Draw_line_vert_3310(i-104, old_point-104, ii, 1);   // Disegna colonna
                if (i <= 8) i = 8;
                 if (i >= 55) i = 55;
                  Draw_line_vert_3310(i-8, old_point-8, ii+1, 1);   // Disegna colonna
                   old_point = i;
             }
              ii ++;
         }
// --------------------------------------------- Marker
        Draw_Mark_Vert ();
       //  Draw_Mark_Hor ();
// --- Delay
        if (Value_Div_Tempo <= 5) Delay_msec(25);
         else  Delay_msec(5); // 5
      }
//
             if (Value_Trigger_Mode == 2) M_trigger = 1;    // TRIGGER ONCE mode
              else M_trigger = 0;
               M_trigger = 0; ////////////////////////////////////////
// --- Campionameto e visualizzazione in tempo reale
 _loop_camp_lungo:
         //   if (M_camp_lungo) {   // RICHIESTA CAMPIONAMENTO DA INTERRUPT (TMR0)
            if (M_t6){
              Campionamento_Lungo ();
              // LED_r ^= 1;
            }
// -----------------------------------------------------
       if (M_repeat) {
         Pulsanti ();
          if (buffer_puls != 5) Page_1_info ();
       }
         Wait_Puls_NOSTOP ();    // Attesa rilascio pulsanti-p
// ---
        if (Value_Trigger_Mode == 2) {    // Once mode
          goto _loop_3;
        }
//
   } // WHILE
//
 } // MAIN
/*













       while (1) {
           Write_String("JUMBO", 1, 10);
            Delay_msec(500);
            Write_Bitmap(1);
               Delay_msec(2000);
             Write_Bitmap(0);
               Delay_msec(2000);
              Write_Bitmap(2);
               Delay_msec(2000);
                nokia_erase_y(0);
                nokia_erase_y(1);
                nokia_erase_y(2);
                nokia_erase_y(3);
                nokia_erase_y(4);
                nokia_erase_y(5);
        }





















*/